#The code is released under GPLv2 (http://www.gnu.org/licenses/gpl-2.0.html),
#by Laszlo Toth.
#Use the code at your own responsibility.
#For help and disclaimer please visit:
#  http://www.soonerorlater.hu/index.khtml?article_id=517

import wx
import sys
from wx.lib.pubsub import Publisher
import  wx.lib.mixins.listctrl  as  listmix

class AutoWidthListCtrl(wx.ListCtrl, listmix.ListCtrlAutoWidthMixin):
    def __init__(self, parent, ID, pos=wx.DefaultPosition,
                 size=wx.DefaultSize, style=0):
        wx.ListCtrl.__init__(self, parent, ID, pos, size, style)
        listmix.ListCtrlAutoWidthMixin.__init__(self)


class ConnectionList(wx.Frame):
    def __init__(self, parent, id, title, server):
        wx.Frame.__init__(self, parent, id, title, size=(760, 460))
        
        self.server=server
        hbox = wx.BoxSizer(wx.VERTICAL)
        panel = wx.Panel(self, -1)

        self.list = AutoWidthListCtrl(panel, -1, style=wx.LC_REPORT)
        self.list.InsertColumn(0, 'Server')
        self.list.InsertColumn(1, 'Port')
        self.list.InsertColumn(2, 'Version')
        self.list.InsertColumn(3, 'Client')
        self.list.InsertColumn(4, 'Port')
        self.list.InsertColumn(5, 'Version')
        self.list.InsertColumn(6, 'User')
        self.list.InsertColumn(7, 'Hijacked')

        self.log=wx.TextCtrl(panel, wx.NewId(), style=wx.TE_MULTILINE|wx.TE_READONLY|wx.HSCROLL)
        stlist=wx.StaticText(panel, -1, "List of the connections:")
        stlog=wx.StaticText(panel, -1, "Log messages:")

        hbox.Add(stlist,0,wx.ALL)
        hbox.Add(self.list, 1, wx.EXPAND)
        hbox.Add(stlog,0,wx.ALL)
        hbox.Add(self.log, 2, wx.EXPAND)
        panel.SetSizer(hbox)
        # create a pubsub receiver
        Publisher().subscribe(self.insertData, "insertData")
        Publisher().subscribe(self.insertLog, "insertLog")
        Publisher().subscribe(self.setHijackPort, "setHijackPort")
        Publisher().subscribe(self.setUser, "setUser")
        
        
        #Popup menu for hijacking
        #for wxMSW
        self.list.Bind(wx.EVT_COMMAND_RIGHT_CLICK, self.OnRightClick)
        #for wxGTK
        self.list.Bind(wx.EVT_RIGHT_UP, self.OnRightClick)

        self.Centre()
        self.Show(True)
        
    def insertData(self, msg):
        data=msg.data.getAsarray()
        self.index = self.list.InsertStringItem(sys.maxint,data[0])
        self.list.SetColumnWidth(0, wx.LIST_AUTOSIZE)
        msg.data.setIndex(self.index)
        for i in range(1,8):
            self.list.SetStringItem(self.index,i,data[i])
            #We do not resize for version and hijack columns for client
            if(i!=7 and i!=5):
                self.list.SetColumnWidth(i, wx.LIST_AUTOSIZE)
            
    def insertLog(self,msg):
        log=msg.data
        self.log.AppendText(log+"\n")
    
    def OnRightClick(self, event):
        if not hasattr(self, "popupID1"):
            self.popupHIJACK = wx.NewId()
        
        self.Bind(wx.EVT_MENU, self.OnPopupHijack, id=self.popupHIJACK)
        
        menu=wx.Menu()
        menu.Append(self.popupHIJACK, "Hijack the connection")
        self.PopupMenu(menu)
        menu.Destroy()
            
    def OnPopupHijack(self,event):
        index=self.list.GetFirstSelected()
        self.server.connections[index].hijacked=True
    
    def setHijackPort(self, msg):
        (index,port)=msg.data
        self.list.SetStringItem(index,7,str(port))
    
    def setUser(self, msg):
        (index,user)=msg.data
        self.list.SetStringItem(index,7,user)
