#The code comes from python-ntlm, so the license should handled accordingly

# This library is free software: you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation, either
# version 3 of the License, or (at your option) any later version.
 
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
# 
# You should have received a copy of the GNU Lesser General Public
# License along with this library. 
# If not, see <http://www.gnu.org/licenses/> or 
#             <http://www.gnu.org/licenses/lgpl.txt>.

import binascii
from struct import *

NTLM_NegotiateUnicode                =  0x00000001
NTLM_NegotiateOEM                    =  0x00000002
NTLM_RequestTarget                   =  0x00000004
NTLM_Unknown9                        =  0x00000008
NTLM_NegotiateSign                   =  0x00000010
NTLM_NegotiateSeal                   =  0x00000020
NTLM_NegotiateDatagram               =  0x00000040
NTLM_NegotiateLanManagerKey          =  0x00000080
NTLM_Unknown8                        =  0x00000100
NTLM_NegotiateNTLM                   =  0x00000200
NTLM_NegotiateNTOnly                 =  0x00000400
NTLM_Anonymous                       =  0x00000800
NTLM_NegotiateOemDomainSupplied      =  0x00001000
NTLM_NegotiateOemWorkstationSupplied =  0x00002000
NTLM_Unknown6                        =  0x00004000
NTLM_NegotiateAlwaysSign             =  0x00008000
NTLM_TargetTypeDomain                =  0x00010000
NTLM_TargetTypeServer                =  0x00020000
NTLM_TargetTypeShare                 =  0x00040000
NTLM_NegotiateExtendedSecurity       =  0x00080000
NTLM_NegotiateExtendedSecurity_0       =  0x00000000
NTLM_NegotiateIdentify               =  0x00100000
NTLM_Unknown5                        =  0x00200000
NTLM_RequestNonNTSessionKey          =  0x00400000
NTLM_NegotiateTargetInfo             =  0x00800000
NTLM_Unknown4                        =  0x01000000
NTLM_NegotiateVersion                =  0x02000000
NTLM_Unknown3                        =  0x04000000
NTLM_Unknown2                        =  0x08000000
NTLM_Unknown1                        =  0x10000000
NTLM_Negotiate128                    =  0x20000000
NTLM_NegotiateKeyExchange            =  0x40000000
NTLM_Negotiate56                     =  0x80000000

# we send these flags with our type 1 message
NTLM_TYPE1_FLAGS = (NTLM_NegotiateUnicode | \
                    NTLM_NegotiateOEM | \
                    NTLM_RequestTarget | \
                    NTLM_NegotiateNTLM | \
                    NTLM_NegotiateOemDomainSupplied | \
                    NTLM_NegotiateOemWorkstationSupplied | \
                    NTLM_NegotiateAlwaysSign | \
                    NTLM_NegotiateExtendedSecurity | \
                    NTLM_NegotiateVersion | \
                    NTLM_Negotiate128 | \
                    NTLM_Negotiate56 )
                    
NTLM_TYPE1_FLAGS_NTLMv1 = (NTLM_NegotiateUnicode | \
    NTLM_NegotiateOEM | \
    NTLM_RequestTarget | \
    NTLM_NegotiateNTLM | \
    NTLM_NegotiateOemDomainSupplied | \
    NTLM_NegotiateOemWorkstationSupplied | \
    NTLM_NegotiateAlwaysSign | \
    #NTLM_NegotiateExtendedSecurity | \
    NTLM_NegotiateVersion | \
    NTLM_Negotiate128 | \
    NTLM_Negotiate56 )


def parse_NTLM_AUTHENTICATE_MESSAGE(data):
    offset=data.find('NTLMSSP')
    if(offset==-1):
        return ("","","")

    msg3=data[offset:len(data)]
    Signature = msg3[0:8]
    msg_type = unpack("<I",msg3[8:12])[0]
    
    if(msg_type!=3):
        return ("","","")
    (LmChallengeResponseOffset,NtChallengeResponseOffset,DomainNameLen,
     DomainNameOffset,UserNameLen, UserNameOffset,
     WorkstationLen, WorkstationOffset)=unpack("8x4x2x2xI2x2xIH2xIH2xIH2xI{0}x".format(len(msg3)-52),msg3)
    LmChallengeResponse,=unpack("{0}x24s{1}x".format(LmChallengeResponseOffset,len(msg3)-LmChallengeResponseOffset-24),msg3)
    NtChallengeResponse,=unpack("{0}x24s{1}x".format(NtChallengeResponseOffset,len(msg3)-NtChallengeResponseOffset-24),msg3)
    UserName,=unpack("{0}x{1}s{2}x".format(UserNameOffset,UserNameLen,len(msg3)-UserNameOffset-UserNameLen),msg3)
    
    return (UserName.decode('utf-16'), binascii.hexlify(LmChallengeResponse),
            binascii.hexlify(NtChallengeResponse))


